"""UCD-like command line interface to rfc1157 objects
"""
from pysnmp.proto import rfc1157
from pysnmpap.cli import error

class VersionMixIn:
    cliUcdOptId = None

class CommunityMixIn:
    cliUcdOptId = '-c:'
    cliUcdOptValName = 'community'

class RequestIdMixIn:
    cliUcdOptId = None

class ErrorStatusMixIn:
    cliUcdOptId = None

class ErrorIndexMixIn:
    cliUcdOptId = None

class VarBindListMixIn:
    cliUcdOptBraces = ('<', '>')

class MessageMixIn:
    def cliUcdGetUsage(self):
        u = self['community'].cliUcdGetUsage()
        if len(self['pdu']):
            u = u + self['pdu'].values()[0].cliUcdGetUsage()
        return u

    def cliUcdGetOptsUsage(self, ident=''):
        u = ident+'%s initialization:' % self.__class__.__name__
        u = u + '\n' + ident + self['community'].cliUcdGetOptsUsage(ident+' ')
        if len(self['pdu']):
            u = u + '\n' + ident + self['pdu'].values()[0].cliUcdGetOptsUsage()
        return u
    
    def cliUcdSetArgs(self, args):
        """Parse C/L args into message components
        """
        args = self['community'].cliUcdSetArgs(args)
        if len(self['pdu']):
            return self['pdu'].values()[0].cliUcdSetArgs(args)
        return args
    
# Trap stuff

class EnterpriseMixIn:
    cliUcdOptId = ''
    cliUcdOptValName = 'enterprise-OID'
    
class AgentAddrMixIn:
    cliUcdOptId = ''
    cliUcdOptValName = 'agent-address'
    
class GenericTrapMixIn:
    cliUcdOptId = ''
    cliUcdOptValName = 'trap-int-id'

class SpecificTrapMixIn:
    cliUcdOptId = ''
    cliUcdOptValName = 'trap-int-id'

class TimeStampMixIn:
    cliUcdOptId = ''

mixInComps = [ (rfc1157.Version, VersionMixIn),
               (rfc1157.Community, CommunityMixIn),
               (rfc1157.RequestId, RequestIdMixIn),
               (rfc1157.ErrorStatus, ErrorStatusMixIn),
               (rfc1157.ErrorIndex, ErrorIndexMixIn),
               (rfc1157.VarBindList, VarBindListMixIn),
               (rfc1157.Message, MessageMixIn),
               (rfc1157.Enterprise, EnterpriseMixIn),
               (rfc1157.AgentAddr, AgentAddrMixIn),
               (rfc1157.GenericTrap, GenericTrapMixIn),
               (rfc1157.SpecificTrap, SpecificTrapMixIn),
               (rfc1157.TimeStamp, TimeStampMixIn) ]

for (baseClass, mixIn) in mixInComps:
    if mixIn not in baseClass.__bases__:
        baseClass.__bases__ = (mixIn, ) + baseClass.__bases__
